@@ -1,10 +1,35 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
-# DANA - Distributed (Asynchronous) Numerical Adaptive computing framework
-# Copyright (C) 2009-2010  Nicolas P. Rougier
+# Copyright INRIA
+# Contributors: Nicolas P. Rougier (Nicolas.Rougier@inria.fr)
 #
-# Distributed under the terms of the BSD License. The full license is in
-# the file COPYING, distributed as part of this software.
+# DANA is a computing framework for the simulation of distributed,
+# asynchronous, numerical and adaptive models.
+#
+# This software is governed by the CeCILL license under French law and abiding
+# by the rules of distribution of free software. You can use, modify and/ or
+# redistribute the software under the terms of the CeCILL license as circulated
+# by CEA, CNRS and INRIA at the following URL
+# http://www.cecill.info/index.en.html.
+#
+# As a counterpart to the access to the source code and rights to copy, modify
+# and redistribute granted by the license, users are provided only with a
+# limited warranty and the software's author, the holder of the economic
+# rights, and the successive licensors have only limited liability.
+#
+# In this respect, the user's attention is drawn to the risks associated with
+# loading, using, modifying and/or developing or reproducing the software by
+# the user in light of its specific status of free software, that may mean that
+# it is complicated to manipulate, and that also therefore means that it is
+# reserved for developers and experienced professionals having in-depth
+# computer knowledge. Users are therefore encouraged to load and test the
+# software's suitability as regards their requirements in conditions enabling
+# the security of their systems and/or data to be ensured and, more generally,
+# to use and operate it in the same conditions as regards security.
+#
+# The fact that you are presently reading this means that you have had
+# knowledge of the CeCILL license and that you accept its terms.
 # -----------------------------------------------------------------------------
 ''' Some useful functions '''
 import numpy as np
@@ -53,12 +78,12 @@ def convolve1d(Z, K, toric=False):
         return convolve(Z,K,mode='constant')
 
     #return convolve(Z,K,mode='wrap')
-    #R = np.convolve(Z, K, 'same')
-    #i0 = 0
-    #if R.shape[0] > Z.shape[0]:
-    #    i0 = (R.shape[0]-Z.shape[0])/2 + 1 - Z.shape[0]%2
-    #i1 = i0+ Z.shape[0]
-    #return R[i0:i1]
+    # R = np.convolve(Z, K, 'same')
+    # i0 = 0
+    # if R.shape[0] > Z.shape[0]:
+    #     i0 = (R.shape[0]-Z.shape[0])/2 + 1 - Z.shape[0]%2
+    # i1 = i0+ Z.shape[0]
+    # return R[i0:i1]
 
 
 
@@ -104,7 +129,7 @@ def convolve2d(Z, K, USV = None, toric=False):
         U,S,V = U.astype(K.dtype), S.astype(K.dtype), V.astype(K.dtype)
     else:
         U,S,V = USV
-    n = (S > 1e-9).sum()
+    n = (S > 1e-12).sum()
 #    n = (S > 0).sum()
     R = np.zeros( Z.shape )
     for k in range(n):
@@ -257,8 +282,12 @@ def convolution_matrix(src, dst, kernel, toric=False):
      [ 6.  9.  6.]
      [ 4.  6.  4.]]
     '''
-
-
+ 
+    # For a toric connection, it is wrong to have a kernel larger
+    # than the source
+ #   if toric:
+ #       shape = np.minimum(np.array(src.shape), np.array(kernel.shape))
+ #       kernel = extract(kernel, shape, np.rint(np.array(kernel.shape)/2.))
 
     # Get non NaN value from kernel and their indices.
     nz = (1 - np.isnan(kernel)).nonzero()
@@ -294,10 +323,17 @@ def convolution_matrix(src, dst, kernel, toric=False):
         dims.append(len(dst.shape)-1)
         for dim in dims:
             i = index[dim]
+
             if toric:
-                z = (indices[dim][dim] - src.shape[dim]//2 + src_indices[dim][i]) % src.shape[dim]
+                z = (indices[dim][dim] - src.shape[dim]//2 +(kernel.shape[dim]+1)%2 + src_indices[dim][i]) % src.shape[dim]
             else:
-                z = (indices[dim][dim] - src.shape[dim]//2 + src_indices[dim][i])
+                z = (indices[dim][dim] - src.shape[dim]//2 +(kernel.shape[dim]+1)%2 + src_indices[dim][i])
+
+            # if toric:
+            #     z = (indices[dim][dim] - src.shape[dim]/2.0 -(kernel.shape[dim]+1)%2 + src_indices[dim][i]) % src.shape[dim]
+            # else:
+            #     z = (indices[dim][dim] - src.shape[dim]/2.0 -(kernel.shape[dim]+1)%2+ src_indices[dim][i])
+
             n = np.where((z >= 0)*(z < src.shape[dim]))[0]
             if dim == 0:
                 nd[dim] = n.copy()
@@ -313,6 +349,7 @@ def convolution_matrix(src, dst, kernel, toric=False):
         dst_index += 1
 
     return sp.coo_matrix( (D,(R,C)), (dst.size,src.size))
+
     #Z = np.zeros((dst.size,src.size))
     #Z[R,C] = D
     #return Z
