@@ -1,11 +1,44 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
-# DANA - Distributed (Asynchronous) Numerical Adaptive computing framework
-# Copyright (C) 2009-2010  Nicolas P. Rougier
+# Copyright INRIA
+# Contributors: Nicolas P. Rougier (Nicolas.Rougier@inria.fr)
 #
-# Distributed under the terms of the BSD License. The full license is in
-# the file COPYING, distributed as part of this software.
+# DANA is a computing framework for the simulation of distributed,
+# asynchronous, numerical and adaptive models.
+#
+# This software is governed by the CeCILL license under French law and abiding
+# by the rules of distribution of free software. You can use, modify and/ or
+# redistribute the software under the terms of the CeCILL license as circulated
+# by CEA, CNRS and INRIA at the following URL: http://www.cecill.info.
+#
+# As a counterpart to the access to the source code and rights to copy, modify
+# and redistribute granted by the license, users are provided only with a
+# limited warranty and the software's author, the holder of the economic
+# rights, and the successive licensors have only limited liability.
+#
+# In this respect, the user's attention is drawn to the risks associated with
+# loading, using, modifying and/or developing or reproducing the software by
+# the user in light of its specific status of free software, that may mean that
+# it is complicated to manipulate, and that also therefore means that it is
+# reserved for developers and experienced professionals having in-depth
+# computer knowledge. Users are therefore encouraged to load and test the
+# software's suitability as regards their requirements in conditions enabling
+# the security of their systems and/or data to be ensured and, more generally,
+# to use and operate it in the same conditions as regards security.
+#
+# The fact that you are presently reading this means that you have had
+# knowledge of the CeCILL license and that you accept its terms.
 # -----------------------------------------------------------------------------
+'''
+A group object represents a multidimensional, homogeneous group of contiguous
+numpy arrays.  An associated data-type object describes the format of each
+element in the group (its byte-order, how many bytes it occupies in memory,
+whether it is an integer or a floating point number, etc.).
+
+A group is very similar to a numpy record array and those not familiar should
+have a look at numpy first.
+'''
 import inspect
 import numpy as np
 from model import Model
@@ -15,7 +48,7 @@ from declaration import Declaration, DeclarationError
 from diff_equation import DifferentialEquation, DifferentialEquationError
 
 
-class GroupError(Exception):
+class GroupException(Exception):
     pass
 
 
@@ -40,7 +73,7 @@ class Group(object):
     * :meth:`dana.empty_like` : Return a empty group with shape and type of input.
     '''
 
-    def __init__(self, shape=(), dtype=float, model=None, fill=0.0):
+    def __init__(self, shape=(), dtype=float, model=None, fill=0.0, base=None):
         '''
         Creates a new group
 
@@ -102,7 +135,8 @@ class Group(object):
         object.__setattr__(self, '_dtype', np.dtype(dtype))
         object.__setattr__(self, '_shape', shape)
         object.__setattr__(self, '_data', {})
-        object.__setattr__(self, '_base', None)
+        object.__setattr__(self, '_saved', {})
+        object.__setattr__(self, '_base', base)
         object.__setattr__(self, '_scalar', None)
         object.__setattr__(self, '_keys', np.dtype(dtype).names)                
         object.__setattr__(self, '_connections', [])
@@ -111,35 +145,45 @@ class Group(object):
 
         for key in self._keys:
             self._data[key] = np.empty(shape=shape,
-                                          dtype=self._dtype[key])
+                                       dtype=self._dtype[key])
             if fill is not None:
                 if type(fill) in [bool, int, float]:
                     self._data[key][...] = fill
                 elif type(fill) is np.ndarray:
-                    self._data[key][...] = fill[key] 
+                    self._data[key][...] = fill[key]
         if type(fill) in [tuple, list]:
             self[...] = fill
-    
-        __default_network__.append(self)
+
+        for key in self._keys:
+            self._saved[key] = self._data[key].copy()
+        for eq in model._declarations:
+            self._saved[eq._varname] = self._data[eq._varname]
+
+        if base is None:
+            __default_network__.append(self)
         try:
-           self.setup()
+            self.setup()
         except:
             pass
 
 
     def setup(self, namespace=None):
-        ''' '''
+        '''
+        '''
         
         # Get unknown constants (and only them) from upper frame
-        if not namespace:
+        if namespace is None:
             namespace = {}
         numpy_ns = dir(np)
         unknown = {}
         for eq in self._model._diff_equations:
+            eq.parse(known_variables = self._keys)
             namespace[eq._varname] = self[eq._varname]
         for eq in self._model._equations:
+            eq.parse(known_variables = self._keys)
             namespace[eq._varname] = self[eq._varname]
         for eq in self._model._declarations:
+            eq.parse()
             namespace[eq._varname] = self[eq._varname]
         for eq in self._model._diff_equations:
             for var in eq._variables:
@@ -148,13 +192,13 @@ class Group(object):
                     unknown[var] = None
         for eq in self._model._equations:
             for var in eq._variables:
-                if var not in numpy_ns and var not in namespace \
+                if var not in numpy_ns and var not in namespace.keys() \
                         and var not in unknown:
                     unknown[var] = None
         for i in range(1,len(inspect.stack())):
             frame = inspect.stack()[i][0]
             for name in unknown:
-                if name in frame.f_globals.keys() and name not in namespace:
+                if name in frame.f_globals.keys(): # and name not in namespace.keys():
                     unknown[name] =  frame.f_globals[name]
                     namespace[name] =  frame.f_globals[name]
         self._namespace = namespace
@@ -172,79 +216,115 @@ class Group(object):
                 for key in source._data.keys():
                     source._data[key] *= source.mask
 
+        # Copy current values to saved ones
+        for key in self._data.keys():
+            self._saved[key][...] = self._data[key]
 
-    def evaluate(self, dt=1):
-        ''' '''
+
+    def propagate(self):
+        for connection in self._connections:
+           connection.propagate()
+
+
+    def evaluate(self, dt=1, update=True):
+        '''
+        Evaluate group state
+
+        **Parameters**
+
+        dt : float
+            Elementary time step
+        update: bool
+            Whether to immediately make computed values public
+        '''
 
         self._namespace['dt'] = dt
-        saved = {}
 
-        for connection in self._connections:
-            connection.propagate()
+        # for connection in self._connections:
+        #    connection.propagate()
+
+        # # Differential equations
+        # for eq in self._model._diff_equations:
+        #     self._saved[eq._varname][...] = self[eq._varname] #.copy()
+        #     self._namespace[eq._varname] = self[eq._varname]
+        # for eq in self._model._diff_equations:
+        #     args = [self._saved[eq._varname],dt]+ \
+        #            [self._namespace[var] for var in eq._variables]
+        #     eq.evaluate(*args)
+        # for eq in self._model._diff_equations:
+        #     self[eq._varname][...] = self._saved[eq._varname]
+
+        # # Equations
+        # for eq in self._model._equations:
+        #     self._saved[eq._varname][...] = self[eq._varname] #.copy()
+        #     self._namespace[eq._varname] = self[eq._varname]
+        # for eq in self._model._equations:
+        #     args = [self._namespace[var] for var in eq._variables]
+        #     self._saved[eq._varname][...] = eq.evaluate(*args) #*self._mask
+        # for eq in self._model._equations:
+        #     self[eq._varname][...] = self._saved[eq._varname]
+
+
+        # # Make sure all masked units are set to 0
+        # if hasattr(self,'mask'):
+        #     for key in self._data.keys():
+        #         self._data[key] *= self.mask
+
 
         # Differential equations
         for eq in self._model._diff_equations:
-            saved[eq._varname] = self[eq._varname].copy()
+            self._saved[eq._varname][...] = self[eq._varname]
             self._namespace[eq._varname] = self[eq._varname]
+
         for eq in self._model._diff_equations:
-            args = [saved[eq._varname],dt]+ \
-                   [self._namespace[var] for var in eq._variables]
+            args = [self._saved[eq._varname],dt]+ \
+                   [self._namespace[var] for var in eq._variables]            
             eq.evaluate(*args)
+
+        # Make newly computed values available to equations below (and only to them)
         for eq in self._model._diff_equations:
-            self[eq._varname][...] = saved[eq._varname]
+            self._namespace[eq._varname] = self._saved[eq._varname]
 
         # Equations
         for eq in self._model._equations:
-            saved[eq._varname] = self[eq._varname].copy()
+            self._saved[eq._varname][...] = self[eq._varname]
             self._namespace[eq._varname] = self[eq._varname]
         for eq in self._model._equations:
             args = [self._namespace[var] for var in eq._variables]
-            saved[eq._varname][...] = eq.evaluate(*args) #*self._mask
-        for eq in self._model._equations:
-            self[eq._varname][...] = saved[eq._varname]
+            self._saved[eq._varname][...] = eq.evaluate(*args) #*self._mask
+            # Make results available to subsequent equations
+            self._namespace[eq._varname] = self._saved[eq._varname]
 
         # Make sure all masked units are set to 0
         if hasattr(self,'mask'):
             for key in self._data.keys():
-                self._data[key] *= self.mask
+                self._saved[key] *= self.mask
 
+        if update:
+            self.update()
+
+
+    def update(self):
+        '''
+        Update group state by making public previously computed values
+        '''
+        for eq in self._model._diff_equations:
+            self._data[eq._varname][...] = self._saved[eq._varname]
+        for eq in self._model._equations:
+            self._data[eq._varname][...] = self._saved[eq._varname]
+        
+
+    def learn(self, dt=1):
         # Learning
         for connection in self._connections:
             connection.evaluate(dt)
 
 
-    def run(self, t=1.0, dt=0.01, n=None):
-        ''' '''
-
-        if n == None:
-            n = int(t/dt)
-        else:
-            dt = 1
-        self.setup()
-        args,saved = {}, {}
-
-        for eq in self._model._diff_equations:
-            args[eq.varname] = [self._data[eq.varname],dt]+ \
-                [self._namespace[var] for var in eq._variables]
-        for eq in self._model._equations:
-            args[eq.varname] = [self._namespace[var] for var in eq._variables]
-        self._namespace['dt'] = dt
-
-        for i in range(int(t/dt)):
-            for connection in self._connections:
-                connection.propagate()
-            for eq in self._model._diff_equations:
-                eq.evaluate(*args[eq.varname])
-            for eq in self._model._equations:
-                self._data[eq.varname][...] = eq.evaluate(*args[eq.varname])
-
-            # Make sure all masked units are set to 0
-            if hasattr(self,'mask'):
-                for key in self._data.keys():
-                    self._data[key][...] *= self.mask
-
-            for connection in self._connections:
-                connection.evaluate(dt)
+    def run(self, dt=1):
+         ''' '''
+         self.propagate()
+         self.evaluate(dt)
+         self.learn(dt)
 
 
     def item(self):
@@ -328,9 +408,9 @@ class Group(object):
         ''' '''
         dtype = []
         dtype.append( (key, self._dtype[key]) )
-        G = Group(shape=self.shape, dtype=dtype)
+        G = Group(shape=self.shape, dtype=dtype, base=self.base or self)
         G.data[key] = self.data[key]
-        G._base = self.base or self
+        #G._base = self.base or self
 
         # Get subgroup relevant connections
         base = G._base
