@@ -1,11 +1,36 @@
 #!/usr/bin/env python
-#-----------------------------------------------------------------------------
-# DANA - Distributed (Asynchronous) Numerical Adaptive computing framework
-# Copyright (C) 2009-2010  Nicolas P. Rougier
+# -*- coding: utf-8 -*-
+# -----------------------------------------------------------------------------
+# Copyright INRIA
+# Contributors: Nicolas P. Rougier (Nicolas.Rougier@inria.fr)
 #
-# Distributed under the terms of the BSD License. The full license is in
-# the file COPYING, distributed as part of this software.
-#-----------------------------------------------------------------------------
+# DANA is a computing framework for the simulation of distributed,
+# asynchronous, numerical and adaptive models.
+#
+# This software is governed by the CeCILL license under French law and abiding
+# by the rules of distribution of free software. You can use, modify and/ or
+# redistribute the software under the terms of the CeCILL license as circulated
+# by CEA, CNRS and INRIA at the following URL
+# http://www.cecill.info/index.en.html.
+#
+# As a counterpart to the access to the source code and rights to copy, modify
+# and redistribute granted by the license, users are provided only with a
+# limited warranty and the software's author, the holder of the economic
+# rights, and the successive licensors have only limited liability.
+#
+# In this respect, the user's attention is drawn to the risks associated with
+# loading, using, modifying and/or developing or reproducing the software by
+# the user in light of its specific status of free software, that may mean that
+# it is complicated to manipulate, and that also therefore means that it is
+# reserved for developers and experienced professionals having in-depth
+# computer knowledge. Users are therefore encouraged to load and test the
+# software's suitability as regards their requirements in conditions enabling
+# the security of their systems and/or data to be ensured and, more generally,
+# to use and operate it in the same conditions as regards security.
+#
+# The fact that you are presently reading this means that you have had
+# knowledge of the CeCILL license and that you accept its terms.
+# -----------------------------------------------------------------------------
 '''
 SparseConnection
 '''
@@ -50,6 +75,12 @@ class SparseConnection(Connection):
         # Else, we need to build it
         elif weights.shape != (self.target.size,self.source.size):
             if len(weights.shape) == len(self.source.shape):
+                # If we have a toric connection, weights cannot be greater than source
+                # in any dimension
+                if self._toric:
+                    s = np.array(self.source.shape)
+                    w = np.array(weights.shape)
+                    weights = extract(weights, np.minimum(s,w), w//2)
                 weights = convolution_matrix(self.source, self.target, weights, self._toric)
             else:
                 raise ConnectionError, \
