@@ -1,10 +1,35 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
-# DANA - Distributed (Asynchronous) Numerical Adaptive computing framework
-# Copyright (C) 2009-2010  Nicolas P. Rougier
+# Copyright INRIA
+# Contributors: Nicolas P. Rougier (Nicolas.Rougier@inria.fr)
 #
-# Distributed under the terms of the BSD License. The full license is in
-# the file COPYING, distributed as part of this software.
+# DANA is a computing framework for the simulation of distributed,
+# asynchronous, numerical and adaptive models.
+#
+# This software is governed by the CeCILL license under French law and abiding
+# by the rules of distribution of free software. You can use, modify and/ or
+# redistribute the software under the terms of the CeCILL license as circulated
+# by CEA, CNRS and INRIA at the following URL
+# http://www.cecill.info/index.en.html.
+#
+# As a counterpart to the access to the source code and rights to copy, modify
+# and redistribute granted by the license, users are provided only with a
+# limited warranty and the software's author, the holder of the economic
+# rights, and the successive licensors have only limited liability.
+#
+# In this respect, the user's attention is drawn to the risks associated with
+# loading, using, modifying and/or developing or reproducing the software by
+# the user in light of its specific status of free software, that may mean that
+# it is complicated to manipulate, and that also therefore means that it is
+# reserved for developers and experienced professionals having in-depth
+# computer knowledge. Users are therefore encouraged to load and test the
+# software's suitability as regards their requirements in conditions enabling
+# the security of their systems and/or data to be ensured and, more generally,
+# to use and operate it in the same conditions as regards security.
+#
+# The fact that you are presently reading this means that you have had
+# knowledge of the CeCILL license and that you accept its terms.
 # -----------------------------------------------------------------------------
 import time
 import numpy as np
@@ -14,9 +39,9 @@ class array(np.ndarray):
     pass
 
 n = 50
-t = 2.0
-dt = 0.00001
-epochs = int(t/dt) #100000
+t = 5.0
+dt = 0.0001
+epochs = int(t/dt)
 
 # Numpy regular array
 Z = np.ones((n,n), dtype=np.double)
@@ -114,5 +139,13 @@ print 'dana differential equation:          ', time.clock()-t0
 group = Group((n,n), 'dV/dt = (V+1) : double')
 group.V = 0
 t0 = time.clock()
-group.run(t,dt)
+for i in range(epochs):
+    group.evaluate(dt=dt)
 print 'dana group:                          ', time.clock()-t0
+
+group.V = 0
+t0 = time.clock()
+eq = group._model._diff_equations[0]
+for i in range(epochs):
+    eq.evaluate(group['V'],dt)
+print 'dana group 2:                        ', time.clock()-t0
